/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-22 15:27:24.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.CoursePaper;
import cn.com.qmth.examcloud.core.print.entity.CourseStatistic;
import cn.com.qmth.examcloud.core.print.enums.PaperStatus;
import cn.com.qmth.examcloud.core.print.repository.CoursePaperRepository;
import cn.com.qmth.examcloud.core.print.repository.CourseStatisticRepository;
import cn.com.qmth.examcloud.core.print.service.CoursePaperService;
import cn.com.qmth.examcloud.core.print.service.bean.coursepaper.CoursePaperConvert;
import cn.com.qmth.examcloud.core.print.service.bean.coursepaper.CoursePaperInfo;
import cn.com.qmth.examcloud.core.print.service.bean.coursepaper.CoursePaperQuery;
import cn.com.qmth.examcloud.core.print.service.bean.coursepaper.CoursePaperTotalInfo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.*;

import static cn.com.qmth.examcloud.core.print.common.Constants.PRT_CODE_500;

/**
 * @author: fengdesheng
 * @since: 2018/10/17
 */
@Service
public class CoursePaperServiceImpl implements CoursePaperService {
    private static final Logger log = LoggerFactory.getLogger(CoursePaperServiceImpl.class);
    @Autowired
    private CoursePaperRepository coursePaperRepository;
    @Autowired
    private CourseStatisticRepository courseStatisticRepository;

    @Override
    public List<CoursePaperInfo> getCoursePaperList(CoursePaperQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getOrgId(), "学校ID不能为空！");
        Check.isNull(query.getExamId(), "考试ID不能为空！");
        Check.isNull(query.getCourseId(), "课程ID不能为空！");

        //查询条件
        SearchBuilder searches = new SearchBuilder();
        searches.eq("orgId", query.getOrgId());
        searches.eq("examId", query.getExamId());
        if (query.getCourseId() != null) {
            searches.eq("courseId", query.getCourseId());
        }
        Specification<CoursePaper> spec = SpecUtils.buildSearchers(CoursePaper.class, searches.build());
        List<CoursePaper> list = coursePaperRepository.findAll(spec);
        return CoursePaperConvert.ofList(list);
    }

    @Override
    public void syncCourseNameByCourseId(Long courseId, String courseName) {
        if (courseId == null || StringUtils.isBlank(courseName)) {
            return;
        }
        coursePaperRepository.updateCourseNameByCourseId(courseId, courseName);
    }

    @Override
    public void syncCoursePaper(CoursePaper info) {
        Check.isNull(info.getOrgId(), "学校ID不能为空！");
        Check.isNull(info.getExamId(), "考试ID不能为空！");
        Check.isNull(info.getCourseId(), "课程ID不能为空！");
        Check.isEmpty(info.getCourseCode(), "课程代码不能为空！");
        Check.isEmpty(info.getCourseName(), "课程名称不能为空！");
        Check.isNull(info.getPaperId(), "试卷ID不能为空！");
        Check.isEmpty(info.getPaperName(), "试卷名称不能为空！");
        Check.isNull(info.getPaperP(), "试卷页数不能为空！");
        Check.isBlank(info.getPaperFileUrl(), "试卷文件地址不能为空！");
        Check.isBlank(info.getAnswerFileUrl(), "答案文件地址不能为空！");

        CoursePaper coursePaper = coursePaperRepository.findByExamIdAndPaperId(info.getExamId(), info.getPaperId());
        if (coursePaper != null) {
            //存在则修改
            coursePaper.setPaperName(info.getPaperName());
            coursePaper.setPaperP(info.getPaperP());
            coursePaper.setPaperFileUrl(info.getPaperFileUrl());
            coursePaper.setAnswerFileUrl(info.getAnswerFileUrl());
            coursePaperRepository.save(coursePaper);
            return;
        }

        //否则新增
        coursePaperRepository.save(info);
    }

    @Override
    public File downloadPaperStructure(Long examId, Long paperId) {
        //todo
        return null;
    }

    @Override
    public void checkPaperStructure(Long examId, Long paperId) {
        //todo
    }

    @Transactional
    public void allotCoursePaper(Long courseStatisticId, Long coursePaperId) {
        Check.isEmpty(courseStatisticId, "课程统计ID不能为空！");
        Check.isEmpty(coursePaperId, "课程试卷ID不能为空！");

        CourseStatistic statistic = courseStatisticRepository.findOne(courseStatisticId);
        Check.isNull(statistic, "当前课程统计不存在！");

        CoursePaper coursePaper = coursePaperRepository.findOne(coursePaperId);
        Check.isNull(coursePaper, "当前课程试卷不存在！");

        statistic.setCoursePaper(coursePaper);
        statistic.setPaperStatus(PaperStatus.已有.getIndex());
        courseStatisticRepository.save(statistic);
    }

    /**
     * 选定学校和考试后，为该考试下的所有待分配课程随机分配试卷
     * 当题库传输过来的试卷数量不足时，会提示试卷数不足的课程名称(代码)
     * 当题库传输过来的试卷数量≥需求时，每种试卷类型随机分配
     * 随机分配后的“课程代码+类型”其试卷状态变为"已有"
     * 试卷状态：已有、无、待指定三种，显示为“待指定”时，点击时弹出试卷制定框
     */
    @Override
    public void allotAllCoursePaper(Long orgId, Long examId) {
        //查询所有未指定试卷的考试课程
        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", orgId)
                .eq("examId", examId)
                .eq("paperStatus", PaperStatus.未指定.getIndex());

        Specification<CourseStatistic> spec = SpecUtils.buildSearchers(CourseStatistic.class, searches.build());
        List<CourseStatistic> statistics = courseStatisticRepository.findAll(spec);
        if (statistics == null || statistics.isEmpty()) {
            log.debug("No course paper need to allot.");
            return;
        }

        //反馈信息
        StringBuffer errors = new StringBuffer();

        //获取考试课程下的试卷列表
        List<CoursePaper> coursePapers = this.getCoursePapers(orgId, examId);

        //按课程封装试卷列表
        Map<Long, List<CoursePaper>> maps = new HashMap<>();
        for (CoursePaper coursePaper : coursePapers) {
            List<CoursePaper> papers = maps.get(coursePaper.getCourseId());
            if (papers == null) {
                papers = new ArrayList<>();
                maps.put(coursePaper.getCourseId(), papers);
            }
            papers.add(coursePaper);
        }

        //按课程随机指定试卷
        List<CourseStatistic> needUpdateStatistics = new ArrayList<>();
        for (CourseStatistic statistic : statistics) {
            List<CoursePaper> papers = maps.get(statistic.getCourseId());
            if (papers == null || papers.isEmpty()) {
                errors.append(String.format("《%s(%s)》", statistic.getCourseName(), statistic.getCourseCode()));
                continue;
            }
            Collections.shuffle(papers);
            statistic.setCoursePaper(papers.get(0));
            statistic.setPaperStatus(PaperStatus.已有.getIndex());
            needUpdateStatistics.add(statistic);
        }

        //批量更新
        if (!needUpdateStatistics.isEmpty()) {
            courseStatisticRepository.save(needUpdateStatistics);
        }

        //试卷不足时，反馈给前端
        if (errors.length() > 0) {
            errors.append("等课程的试卷数不足！");
            throw new StatusException(PRT_CODE_500, errors.toString());
        }
    }

    private List<CoursePaper> getCoursePapers(Long orgId, Long examId) {
        SearchBuilder searches = new SearchBuilder();
        searches.eq("orgId", orgId);
        searches.eq("examId", examId);
        Specification<CoursePaper> spec = SpecUtils.buildSearchers(CoursePaper.class, searches.build());
        return coursePaperRepository.findAll(spec);
    }

    @Override
    public CoursePaperTotalInfo getPaperTotalByOrgIdAndExamId(Long orgId, Long examId) {
        //todo
        return null;
    }

    @Override
    public File exportAllByOrgIdAndExamId(Long orgId, Long examId) {
        //todo
        return null;
    }

    @Override
    public File exportBatchByIds(Long[] ids) {
        //todo
        return null;
    }

}